//
{
  // David Waters, 03.07.2001.
  // =========================
  // Macro for comparing generator level diboson distributions between
  // MCFM and PYTHIA.

  cout << " PYTHIA-MCFM Comparison " << endl;
  
  // Clean up :
  gROOT->Reset();
  gStyle->SetFillColor(0);
  gROOT->SetStyle("Plain");

  // Make a canvas to plot the histograms on :
  c1 = new TCanvas("c1","PYTHIA_MCFM",0,0,600,700);
  // c1->SetCanvasSize(750.,900.);
  c1->Size(0.,10.);

  // Make a postscipt object :
  TPostScript *ps = new TPostScript("pythia_mcfm.ps",111);
  ps->Range(18.5,26.0);  
  ps->NewPage();

  // Make the pads :
  int num_plots_x = 2;
  int num_plots_y = 3;
  int total_pads = num_plots_x*num_plots_y;
  TPad* padPointers[total_pads];
  for (int i = 0; i < num_plots_x; ++i)
    {
      for (int j = 0; j < num_plots_y; ++j)
	{
	  int xindex = i+1;
	  int yindex = j+1;	  int padIndex = num_plots_x * (yindex-1) + xindex - 1;
	  // cout << "padIndex = " << padIndex << endl;
	  padPointers[padIndex] = new TPad("pad1"," ",
					   ((1.0*i)/(1.0*num_plots_x)),
					   (((1.0*num_plots_y-1.0)/(1.0*num_plots_y))-((1.0*j)/(1.0*num_plots_y))),
					   ((1.0*i+1.0)/(1.0*num_plots_x)),
					   (1.0-((1.0*j)/(1.0*num_plots_y))),
					   21);
	  // cout << "lower-left x = " << ((1.0*i)/(1.0*num_plots_x)) << endl;
	  // cout << "lower-left y = " << (((1.0*num_plots_y-1.0)/(1.0*num_plots_y))-((1.0*j)/(1.0*num_plots_y))) << endl;
	  // cout << "upper-right x = " << ((1.0*i+1.0)/(1.0*num_plots_x)) << endl;
	  // cout << "upper-right y = " << (1.0-((1.0*j)/(1.0*num_plots_y))) << endl;

	  padPointers[padIndex]->SetFillColor(0);
	  padPointers[padIndex]->SetGrid();
	  // Get rid of irritating red line at the border :
	  padPointers[padIndex]->SetBorderMode(0.);
	  padPointers[padIndex]->Draw();
	}
    }

  // --------------------------------------------------------------------------
  // Open the file containing the PYTHIA histograms :
  TFile pythiaFile("validation_histos.root");
  pythiaFile->ls();
  // Open the file containing the MCFM histograms :
  TFile mcfmFile("mcfm_histos.root");
  mcfmFile->ls();
  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  // Grab the histograms that are to be plotted :
  // ============================================
  pythiaFile->cd();
  TH1F* pythia_im34 = (TH1F*) gROOT.FindObject("McValidation_massB1");
  TH1F* pythia_im56 = (TH1F*) gROOT.FindObject("McValidation_massB2");
  TH1F* pythia_im3456 = (TH1F*) gROOT.FindObject("McValidation_massB12");
  TH1F* pythia_pt34 = (TH1F*) gROOT.FindObject("McValidation_ptB1");
  TH1F* pythia_pt56 = (TH1F*) gROOT.FindObject("McValidation_ptB2");
  TH1F* pythia_pt3456 = (TH1F*) gROOT.FindObject("McValidation_ptB12");

  mcfmFile->cd();
  TH1F* mcfm_im34 = (TH1F*) gROOT.FindObject("im34");
  TH1F* mcfm_im56 = (TH1F*) gROOT.FindObject("im56");
  TH1F* mcfm_im3456 = (TH1F*) gROOT.FindObject("im3456");
  TH1F* mcfm_pt34 = (TH1F*) gROOT.FindObject("pt34");
  TH1F* mcfm_pt56 = (TH1F*) gROOT.FindObject("pt56");
  TH1F* mcfm_pt3456 = (TH1F*) gROOT.FindObject("pt3456");
  // --------------------------------------------------------------------------

  padPointers[0]->cd();
  padPointers[0]->SetLogy();
  mcfm_im34->SetLineWidth(3);
  mcfm_im34->Draw();
  pythia_im34->SetLineWidth(3);
  pythia_im34->SetLineColor(2);
  pythia_im34->SetLineStyle(2);
  pythia_im34->Draw("same");

  padPointers[1]->cd();
  mcfm_pt34->SetLineWidth(3);
  mcfm_pt34->Draw();
  pythia_pt34->SetLineWidth(3);
  pythia_pt34->SetLineColor(2);
  pythia_pt34->SetLineStyle(2);
  pythia_pt34->Draw("same");

  padPointers[2]->cd();
  padPointers[2]->SetLogy();
  mcfm_im56->SetLineWidth(3);
  mcfm_im56->Draw();
  pythia_im56->SetLineWidth(3);
  pythia_im56->SetLineColor(2);
  pythia_im56->SetLineStyle(2);
  pythia_im56->Draw("same");

  padPointers[3]->cd();
  mcfm_pt56->SetLineWidth(3);
  mcfm_pt56->Draw();
  pythia_pt56->SetLineWidth(3);
  pythia_pt56->SetLineColor(2);
  pythia_pt56->SetLineStyle(2);
  pythia_pt56->Draw("same");

  padPointers[4]->cd();
  padPointers[4]->SetLogy();
  mcfm_im3456->SetLineWidth(3);
  mcfm_im3456->Draw();
  pythia_im3456->SetLineWidth(3);
  pythia_im3456->SetLineColor(2);
  pythia_im3456->SetLineStyle(2);
  pythia_im3456->Draw("same");

  padPointers[5]->cd();
  mcfm_pt3456->SetLineWidth(3);
  mcfm_pt3456->Draw();
  pythia_pt3456->SetLineWidth(3);
  pythia_pt3456->SetLineColor(2);
  pythia_pt3456->SetLineStyle(2);
  pythia_pt3456->Draw("same");


  c1->Update();
  ps->Close();
}

