      subroutine fill_hepeup()
      implicit none

c ----------------------------------------------------------------------
c Dave Waters, 24.07.2002
c 
c Pythia v6.2 user event routine.
c     
c ----------------------------------------------------------------------
c        1         2         3         4         5         6         7

c --- MCFM includes :
#include "mcfm/../src/Inc/constants.F"
#include "mcfm/../src/Inc/npart.F"
#include "mcfm/../src/Inc/stdhep.F"
#include "mcfm/../src/Inc/masses.F"

c --- Pythia common block with particle information :
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500)
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)
      SAVE /HEPEUP/

c --- MCFM common block containing a single event
c --- [Should be a proper include]
      double precision p(10,4)
      double precision wt 
      integer pflav,pbarflav     
      common /cdfevent/p,wt,pflav,pbarflav
c --- MCFM common blocks :
      integer nproc
      common/nproc/nproc
      character*2 plabel(mxpart)
      common/plabel/plabel

c --- Random number for flavour sampling :
      integer noran
      parameter (noran=1)
      real evtran(1)

      integer i,j,stdhep_id,jetlabel_to_stdhep,lutran

      double precision mass,ptwp,ptwm

c ----------------------------------------------------------------------

c --- ******************************************************************
c --- PART I
c --- ======
c --- This is the generic piece that should work for any MCFM
c --- process codes, at least those that are qqbar -> X. Only tested
c --- for WW->evev so far.
c --- ******************************************************************

c --- Two incoming quarks as well as final state particles.
      NUP=npart+2

c --- No intermediate states are put in.
c --- Initial quark and anti-quark :
      ISTUP(1)=-1
      ISTUP(2)=-1
c --  All others are outgoing final state particles :
      do i=3,NUP
        ISTUP(i)=1
      enddo

c --- ******************************************************************
c --- MCFM numbering scheme :
c --- -5  bbar
c --- -4  cbar
c --- -3  sbar
c --- -2  ubar 
c --- -1  dbar 
c ---  0  gluon
c ---  1  d
c ---  2  u
c ---  3  s
c ---  4  c
c ---  5  b
      if (pflav.eq.0) then
        IDUP(1)=21
      else
        IDUP(1)=pflav
      endif

      if (pbarflav.eq.0) then
        IDUP(2)=21
      else
        IDUP(2)=pbarflav
      endif
      
c     write(6,*) 'Flavour combination : IDUP(1), IDUP(2) = ',IDUP(1),
c    +     ' , ',IDUP(2)
c --- ******************************************************************

c --- Get the colour flow correct. Quarks carry colour and
c --- anti-quarks carry anti-colour :
      if (IDUP(1).GT.0) then
c ---   Quark in first place, anti-quark in second :
        ICOLUP(1,1)=102
        ICOLUP(2,1)=0
        ICOLUP(1,2)=0
        ICOLUP(2,2)=102
      else
c ---   Anti-quark in first place, quark in second :
        ICOLUP(1,1)=0
        ICOLUP(2,1)=102
        ICOLUP(1,2)=102
        ICOLUP(2,2)=0
      endif

c --- Fill in the remaning ID's :
      do i=3,NUP
        stdhep_id = jetlabel_to_stdhep(plabel(i))
        IDUP(i)=lutran(stdhep_id,2)
      enddo    

c --- Now actually fill with 4-vectors :
      do i=1,NUP
        do j=1,3
c ---     Parity invert the first two particles ONLY (becuase 
c ---     computationally, MCFM treats them as outgoing
c ---     particles). After doing this, the quark in the 
c ---     first position will be travelling in the +z direction,
c ---     corresponding to the incoming proton direction.
          PUP(j,i)= p(i,j)
          if (i.eq.1 .or. i.eq.2) then
            PUP(j,i) = -PUP(j,i)
          endif
        enddo
        mass=0d0
        if ((plabel(i) .eq. 'el').or.(plabel(i) .eq. 'ea')) then
          mass=mel
        elseif ((plabel(i) .eq. 'ml').or.(plabel(i) .eq. 'ma')) then
          mass=mmu
        elseif ((plabel(i) .eq. 'tl').or.(plabel(i) .eq. 'ta')) then
          mass=mtau
        elseif ((plabel(i) .eq. 'bq').or.(plabel(i) .eq. 'ba')) then
          mass=mb
        endif
        PUP(4,i) = dsqrt(p(i,4)**2+mass**2)
        PUP(5,i) = mass
      enddo
      
c     do i=1,NUP
c       write(6,*) 'PUP(1,',i,') = ',PUP(1,i)
c       write(6,*) 'PUP(2,',i,') = ',PUP(2,i)
c       write(6,*) 'PUP(3,',i,') = ',PUP(3,i)
c       write(6,*) 'PUP(4,',i,') = ',PUP(4,i)
c       write(6,*) 'PUP(5,',i,') = ',PUP(5,i)
c     enddo



c --- ******************************************************************
c --- PART II
c --- =======
c --- For certain process codes, insert additional lines into the 
c --- record.
c --- ******************************************************************

      if (nproc.eq.61) then
c ---   WW->evev
c ---   ========
c ---   Insert resonance entries for the W+ and W-. Amongst other 
c ---   things, this makes navigation of the resulting HEPG bank
c ---   easier for generator level studies.
c ---   Add two more entries :
        NUP=NUP+2
c ---   Shift up by two all the final state particles :
        do i=NUP-2,3,-1
          ISTUP(i+2)=ISTUP(i)
          IDUP(i+2)=IDUP(i)
          do j=1,5
            PUP(j,i+2)=PUP(j,i)
          enddo
        enddo
c ---   Now add the W+ in position 3. ISTUP of 2 indicates that the mass
c ---   should be preserved through any showering.
        ISTUP(3)=2        
        IDUP(3)=24

c ---   Four momentum of the W+ from the decay products :
        PUP(1,3) = PUP(1,5) + PUP(1,6)
        PUP(2,3) = PUP(2,5) + PUP(2,6)
        PUP(3,3) = PUP(3,5) + PUP(3,6)
        PUP(4,3) = PUP(4,5) + PUP(4,6)

        PUP(5,3) = dsqrt(PUP(4,3)**2 - PUP(1,3)**2 - 
     +       PUP(2,3)**2 - PUP(3,3)**2)

c ---   W comes from primary interaction : 
        MOTHUP(1,3) = 1
        MOTHUP(2,3) = 2

c ---   The decay products should point back to the W :
        MOTHUP(1,5) = 3
        MOTHUP(2,5) = 0
        MOTHUP(1,6) = 3
        MOTHUP(2,6) = 0

c ---   Exactly the same for the W- :
        ISTUP(4)=2        
        IDUP(4)=-24 
       
        PUP(1,4) = PUP(1,7) + PUP(1,8)
        PUP(2,4) = PUP(2,7) + PUP(2,8)
        PUP(3,4) = PUP(3,7) + PUP(3,8)
        PUP(4,4) = PUP(4,7) + PUP(4,8)
        
        PUP(5,4) = dsqrt(PUP(4,4)**2 - PUP(1,4)**2 - 
     +       PUP(2,4)**2 - PUP(3,4)**2)
        
        MOTHUP(1,4) = 1
        MOTHUP(2,4) = 2
        
        MOTHUP(1,7) = 4
        MOTHUP(2,7) = 0
        MOTHUP(1,8) = 4
        MOTHUP(2,8) = 0

c ---   Set the scale Q. Supposed to be the scale used for evaluation of PDF's,
c ---   but also used as the maximum virtuality in the initial state parton 
c ---   shower development. Larger values can boost the high pT region (i.e.
c ---   large subsystem pT's generated through parton showers).
c ---   Pythia internal WW generation uses the square root of the average 
c ---   squared transverse mass of the two W's. Note that it also uses this 
c ---   scale for the PDF evaluation, which is NOT the case for MCFM.
c ---   However setting it as below does markedly improve the agreement
c ---   in the pT(WW) distribution between MCFM+showering and Pythia internal
c ---   WW generation.
c ---   The formula on page 189 of the Pythia 6.2 manual, in the entry for
c ---   MSTP(32), is :
c ---   Q^2 = p_T^2 + (m_3^2 + m_4^2)/2 
        ptwp = sqrt(PUP(1,3)**2 + PUP(2,3)**2)
        ptwm = sqrt(PUP(1,4)**2 + PUP(2,4)**2)
c ---   These two pT's should be the same, so :
        SCALUP = sqrt(ptwp**2 + (PUP(5,3)**2 + PUP(5,4)**2)/2.0)

      endif

      return
      end

